// AP(r) Computer Science Marine Biology Simulation:  Class
// added by Georgia Tech to extend past Fish
//
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.util.Random;
import java.awt.Color;
import java.util.ArrayList;

/**
 * Class that represents an abstract drawable
 * object (something that has an id, location,
 * environment, direction and color) and can
 * be drawn.
 * 
 * This class was created
 * to show how to extend the Marine Biology Case
 * Study past Fish. 
 * 
 * @author Barbara Ericson ericson@cc.gatech.edu
 * Georgia Tech Jan 2005
 */
public abstract class AbstractDrawable implements Drawable
{
  //////////////// fields //////////////////////////
  
  // Instance Variables: Encapsulated data for EACH object
  private Color color = Color.YELLOW; // color for this obj
  private Environment env;        // environment in which this object lives
  private int id;                  // unique ID for this object
  private Location loc;            //  location
  private Direction dir;           // direction
  
  //////////////// constructors ////////////////////
  
  /**
   * Constructor that takes an environment and a location
   * @param theId the unique ID for this object
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   */
  public AbstractDrawable(int theId,
                            Environment theEnv, 
                            Location theLoc)
  {
    // call the other constructor with a random direction
    this(theId, theEnv,theLoc,theEnv.randomDirection());
  }
  
  /**
   * Constructor that takes an environment, location, and 
   * direction
   * @param theId the unique ID for this object
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   * @param theDir the direction this object is facing
   */
  public AbstractDrawable(int theId,
                            Environment theEnv, 
                            Location theLoc, 
                            Direction theDir)
  {
    id = theId;   // set the id
    env = theEnv; // set the environment
    loc = theLoc; // set the location
    dir = theDir; // set the direction
    env.add(this); // add this object to the environment
  }
  
  /**
   * Constructor that takes an environment, location, and 
   * direction
   * @param theId the unique ID for this object
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   * @param theDir the direction this object is facing
   * @param theColor the new color to use
   */
  public AbstractDrawable(int theId,
                            Environment theEnv, 
                            Location theLoc, 
                            Direction theDir,
                            Color theColor)
  {
    this(theId,theEnv,theLoc,theDir);
    color = theColor;
  }
  
  //////////////// methods /////////////////////////
  
   /** Returns a string representing key information about this fish.
     *  @return  a string indicating the fish's ID, location, and direction
     **/
    public String toString()
    {
        return getClass() + " " + id() + location() + 
          direction();
    }
  
   /** Returns this fish's ID.
     *  @return        the unique ID for this fish
     **/
    public int id()
    {
        return id;
    }
  
  /**
   * Method to return the color of this object
   * @return the color of the object
   */
  public Color color() { return color; }
  
  /**
   * Method to set the color of this object
   * @param theColor the color to use
   */
  public void setColor(Color theColor) 
  {
    color = theColor;
  }
  
  /** Checks whether this object is in an environment.
   *  @return  <code>true</code> if the object is in the 
   *  environment (and at the correct location); 
   *  <code>false</code> otherwise
   **/
  public boolean isInEnv()
  {
    return environment().objectAt(location()) == this;
  }
  
  /** 
   * Returns this objects location.
   * @return  the location of this object in the 
   * environment
   **/
  public Location location()
  {
    return loc;
  }
  
  /**
   * Method to change the location
   * @param newLoc the new location
   */
  public void setLocation(Location newLoc)
  { 
    loc = newLoc;
  }
  
  
  /** Returns this object's direction.
   *  @return the direction in which this object is facing
   **/
  public Direction direction()
  {
    return dir;
  }
  
  /** Returns this object's environment.
   *  @return the environment in which this object lives
   **/
  public Environment environment()
  {
    return env;
  }
  
  /** Modifies this object's direction.
   *  @param  newDir    new direction value
   **/
  public void changeDirection(Direction newDir)
  {
    // Change direction.
    dir = newDir;
  }
  
}
