// AP(r) Computer Science Marine Biology Simulation:
//
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.util.Random;
import java.awt.Color;
import java.util.ArrayList;

/**
 * Class that represents a mammal.  These
 * animals need to surface to breathe 
 * (They are not fish).  
 * 
 * This class was created
 * to show how to extend the Marine Biology Case
 * Study past Fish. 
 * 
 * @author Barbara Ericson ericson@cc.gatech.edu
 * Georgia Tech Jan 2005
 */
public abstract class Mammal extends AbstractActionable
{
  //////////////// fields //////////////////////////
  
  // time since last breath
  private int stepsSinceBreath = 0;
  
  // how long this can last till it has to breathe or die
  private static int maxStepsBetweenBreaths = 20;
  
  //////////////// constructors ////////////////////
  
  /**
   * Constructor that takes an id, environment and location
   * @param theId   the unique id for this object
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   */
  public Mammal(int theId, Environment theEnv, 
                Location theLoc)
  {
    // call the other constructor with a random direction
    super(theId, theEnv,theLoc,theEnv.randomDirection());  
  }
  
  /**
   * Constructor that takes an id, environment, location,
   * and the maximum number of steps between breaths
   * @param theId   the unique id for this object
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   * @param maxSteps the max # of steps between breaths
   */
  public Mammal(int theId, Environment theEnv, 
                Location theLoc, int maxSteps)
  {
    // call the other constructor with a random direction
    super(theId, theEnv,theLoc,theEnv.randomDirection());
    maxStepsBetweenBreaths = maxSteps;
  }
  
   /**
   * Constructor that takes the id, environment, location,
   * and direction
   * @param theId   the unique id for this object
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   * @param theDir the direction this object is facing
   */
  public Mammal(int theId, Environment theEnv, 
                Location theLoc, Direction theDir)
  {
    // let the parent constructor handle this
    super(theId,theEnv,theLoc,theDir);
  }
  
  /**
   * Constructor that takes the id, environment, location,
   * direction, and max number of steps between breaths
   * @param theId   the unique id for this object
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   * @param theDir the direction this object is facing
   * @param maxSteps the max # of steps between breaths
   */
  public Mammal(int theId, Environment theEnv, 
                Location theLoc, Direction theDir,
                int maxSteps)
  {
    // let the parent constructor handle this
    super(theId,theEnv,theLoc,theDir);
    maxStepsBetweenBreaths = maxSteps;
  }
  
  /**
   * Constructor that takes the id, environment, location, 
   * direction, and color
   * @param theId   the unique id for this object
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   * @param theDir the direction this object is facing
   * @param theColor the color to use
   */
  public Mammal(int theId, Environment theEnv, 
                Location theLoc, Direction theDir, 
                Color theColor)
  {
    // let the parent constructor handle this
    super(theId,theEnv,theLoc,theDir);
    setColor(theColor);
  }
  
   /**
   * Constructor that takes the id, environment, location, 
   * direction, color, and max number of steps between
   * breaths
   * @param theId   the unique id for this object
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   * @param theDir the direction this object is facing
   * @param theColor the color to use
   * @param maxSteps the max # of steps between breaths
   */
  public Mammal(int theId, Environment theEnv, 
                Location theLoc, Direction theDir, 
                Color theColor, int maxSteps)
  {
    // let the parent constructor handle this
    super(theId,theEnv,theLoc,theDir);
    setColor(theColor);
    maxStepsBetweenBreaths = maxSteps;
  }
  
  //////////////// methods /////////////////////////
  
  /**
   * Method to tell the object to act
   */
  public void act()
  {
    stepsSinceBreath++;
    
    // check that this object is still in the environment
    if (isInEnv())
    {
      
      // check if too long since breath
      if (stepsSinceBreath > maxStepsBetweenBreaths)
        die();
      else
        // try to move
        move();
    }
  }
  
  /**
   * Method to try to move this object
   */
  public void move()
  {
    // get the current y
    int currRow = location().row();
    if (currRow == 0)
      stepsSinceBreath = 0;
    
    // declare variables
    Location[] locArray = null;
    Location nextLoc = null;
    
    // if will need to breathe soon try to rise
    if ((maxStepsBetweenBreaths - stepsSinceBreath) <= 
        (currRow + 2))
    {
      Debug.println("going north");
      locArray = getNorthEmptyNeighbors();
    }
    
    // else go south, southeast, or southwest
    else
    {
      locArray = getSouthEmptyNeighbors();
      Debug.println("going south");
    }
    
    // if no empty neighbors stay where you are
    if (locArray[0] == null)
    {
      Debug.println("no empty neighbors");
      nextLoc = location();
    }
    // take first empty neighbor
    else
    {
      Debug.println("are empty neighbors");
      nextLoc = locArray[0];
    }
    
    // move to the next location
    move(nextLoc);
  }
}
